package it.dsestili.jhash.core;

/*
JHash a simple hash code generator
Copyright (C) 2013 Davide Sestili

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import it.dsestili.jhash.gui.InputPanel;
import it.dsestili.jhash.gui.StatusPanel;
import java.awt.Component;
import java.io.File;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

public class WorkerThread extends Thread implements IProgressListener
{
	protected Core core;
	protected final StatusPanel statusPanel;
	protected final InputPanel inputPanel;
	protected final JButton btnStart;
	protected final File file;
	protected final String algorithm;

	public WorkerThread(File file, String algorithm, StatusPanel statusPanel, InputPanel inputPanel, JButton btnStart)
	{
		this.btnStart = btnStart;
		this.inputPanel = inputPanel;
		this.statusPanel = statusPanel;
		this.file = file;
		this.algorithm = algorithm;
	}

	public void interruptOperation()
	{
		if(core != null)
		{
			core.interrupt();
		}
	}

	@Override
	public void run() 
	{
		try
		{
			core = new Core(file, algorithm);
			core.addIProgressListener(this);

			manageGUI(false);

			String hash = core.generateHash();
			
			showHash(hash);
		}
		catch(InterruptedException e)
		{
			showStatus("Operation canceled");
		}
		catch(Throwable t)
		{
			showStatus("Error occurred");
			
			t.printStackTrace();
		}
		finally
		{
			manageGUI(true);
		}
	}

	protected void manageGUI(final boolean enabled)
	{
		SwingUtilities.invokeLater(new Runnable() 
		{
			@Override
			public void run() 
			{
				inputPanel.setEnabled(enabled);
				
				for(Component component : inputPanel.getComponents())
				{
					component.setEnabled(enabled);
				}

				btnStart.setText(enabled ? "Generate Hash" : "Cancel");
			}
		});
	}

	private void showHash(final String hash)
	{
		SwingUtilities.invokeLater(new Runnable() 
		{
			@Override
			public void run() 
			{
				inputPanel.setGeneratedHash(hash);
			}
		});
		
		try 
		{
			int result = JOptionPane.showConfirmDialog(null, "Save hash to a file?", "Save hash", JOptionPane.YES_NO_OPTION);
			if(result == JOptionPane.YES_OPTION)
			{
				Vector<Vector<String>> hashList = new Vector<Vector<String>>();
				
				Vector<String> info = new Vector<String>();
				info.add(file.getName());
				info.add(hash);
				
				hashList.add(info);
			
				Utils.saveHashToFile(null, hashList);
			}
		}
		catch(Throwable e) 
		{
			JOptionPane.showMessageDialog(null, "Error occurred", "Error occurred", JOptionPane.ERROR_MESSAGE);
			
			e.printStackTrace();
		}
	}

	protected void showStatus(final String status)
	{
		SwingUtilities.invokeLater(new Runnable() 
		{
			@Override
			public void run() 
			{
				statusPanel.setStatus(status);
			}
		});
	}
	
	@Override
	public void progressEvent(final ProgressEvent event) 
	{
		SwingUtilities.invokeLater(new Runnable() 
		{
			@Override
			public void run() 
			{
				statusPanel.setStatus(event.toString());
				statusPanel.setValue(event.getPCent());
			}
		});
	}
	
}
